/***************************************************************************
 $RCSfile: createuser.cpp,v $
                             -------------------
    cvs         : $Id: createuser.cpp,v 1.9 2003/01/21 02:33:49 aquamaniac Exp $
    begin       : Sat Jan 26 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 */


#include "createuser.h"


#ifdef HAS_LIBCHIPCARD
int openCard(HBCI::Pointer<CTCard> card) {
  bool b;
  unsigned int rflags;
  unsigned int rstatus;
  CTError err;

  fprintf(stderr,"Please insert your HBCI card into any reader\n");
  rstatus=CHIPCARD_STATUS_INSERTED | CHIPCARD_STATUS_PROCESSOR;
  rflags=0;
  b=card.ref().waitForCard(1000,
			   false,
			   rflags,
			   0,
			   rstatus,
			   CHIPCARD_STATUS_INSERTED|
			   CHIPCARD_STATUS_PROCESSOR|
			   CHIPCARD_STATUS_LOCKED_BY_OTHER);
  if (!b) {
    fprintf(stderr,"No card inserted within some seconds, aborting");
    return 2;
  }
  fprintf(stderr,"Card is inserted, working.\n");
  err=card.ref().openCard();
  if (!err.isOk()) {
    fprintf(stderr,"%s\n",err.errorString().c_str());
    return 3;
  }
  return 0;
}


MediumType checkCardType() {
  // we have to check for the card type (DDV or RSA)
  HBCI::Pointer<CTCard> card;
  HBCI::Pointer<HBCICard> hbcicard;
  HBCI::Pointer<RSACard> rsacard;
  CTError err;
  int rv;
  MediumType mtype;

  card=new CTCard();
  rv=openCard(card);
  if (rv!=0) {
    throw HBCI::Error("checkCardType()",
		      ERROR_LEVEL_NORMAL,
		      rv+0x1000,
		      ERROR_ADVISE_ABORT,
		      "could not open card");
  }
  hbcicard=new HBCICard(card.ref());
  card=0;
  err=hbcicard.ref().reopenCard();
  if (err.isOk()) {
    mtype=MediumDDVCard;
    hbcicard.ref().closeCard();
    hbcicard=0;
  }
  else {
    fprintf(stderr,"Error: %s\n",err.errorString().c_str());
    rsacard=new RSACard(hbcicard.ref());
    hbcicard=0;
    err=rsacard.ref().reopenCard();
    if (err.isOk()) {
      mtype=MediumRDHCard;
      rsacard.ref().closeCard();
      rsacard=0;
    }
    else {
      fprintf(stderr,"Sorry, this is neither a RSA nor a DDV card.\n");
      rsacard.ref().closeCard();
      rsacard=0;
      throw HBCI::Error("checkCardType()",
			ERROR_LEVEL_NORMAL,
			0,
			ERROR_ADVISE_ABORT,
			"no HBCI card");
    }
  }
  return mtype;
}


#endif // HAS_LIBCHIPCARD



HBCI::Error reallyCreateUser(HBCI::Pointer<HBCI::API> hbciif,
			     const string &mediumname,
			     MediumType mt,
			     int country,
			     const string &instid,
			     const string &userid,
			     const string &customerid,
			     const string &role,
			     const string &server) {
  HBCI::Pointer<HBCI::Bank> bank;
  HBCI::Pointer<HBCI::User> user;
  HBCI::Pointer<HBCI::Customer> cust;
  HBCI::Pointer<HBCI::Medium> medium;
  HBCI::Error err;

  // create medium
  fprintf(stderr,"instid=%s,userid=%s, server=%s\n",
	  instid.c_str(),userid.c_str(),server.c_str());
  if (country==0 ||
      instid.empty() ||
      userid.empty() ||
      server.empty())
    return HBCI::Error("realyCreateUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_DONTKNOW,
		       "empty arguments");

  // create user and medium
  try {
    // check if the bank exists
    bank=hbciif.ref().findBank(country,instid);
    if (!bank.isValid()) {
      // need to create it
      bank=hbciif.ref().bankFactory(country,
				    instid,
				    server);
      hbciif.ref().addBank(bank);
    }

    // check if user exists
    user=bank.ref().findUser(userid);
    if (!user.isValid()) {
      // need to create him

      // check if medium exists
      medium=hbciif.ref().findMedium(mediumname);
      if (!medium.isValid())
	// need to create the medium
	medium=hbciif.ref().createNewMedium(country,
					    instid,
					    userid,
					    mediumname,
					    mt);
      // then the user himself
      user= hbciif.ref().userFactory(bank,
				     medium,
				     userid);
      // add customer to the system
      bank.ref().addUser(user);

      // done
    }
    // create customer, if given
    if (!customerid.empty()) {
      cust=hbciif.ref().customerFactory(user,
					customerid,
					role);
      user.ref().addCustomer(cust);
    }

  } // try
  catch (HBCI::Error lerr) {
    err=lerr;
  }

  return err;
}


HBCI::Error importUser(HBCI::Pointer<HBCI::API> hbciif,
		       string mediumname,
		       MediumType mtype,
		       int country,
		       const string &instid,
		       const string &userid,
		       const string &customerid,
		       const string &role,
		       const string &server) {
  string tmp;
  HBCI::Pointer<HBCI::Medium> medium;
  int i;
  int j;
  int lcountry;
  string linstid;
  string luserid;
  string lserver;
  HBCI::Error err;

  // now import the contexts from the medium
  medium=hbciif.ref().mediumFactory(mediumname,mtype);
  try {
    i=1;
    j=0;
    err=medium.ref().mountMedium(0);
    if (!err.isOk())
      return HBCI::Error("importUser",err);
    while(1) {
      // get context
      if (!medium.ref().getContext(i,
				   lcountry,
				   linstid,
				   luserid,
				   lserver))
	break;
      // add context, if it matches the criteria
      if (
	  (!country || lcountry==country) &&
	  (-1!=HBCI::parser::cmpPattern(linstid,
					instid,
					false) || instid.empty())
	 ) {
	mediumname=medium.ref().mediumName();
	err=reallyCreateUser(hbciif,
			     mediumname,
			     mtype,
			     lcountry,
			     linstid,
			     userid.empty()?luserid:userid,
			     customerid,
			     role,
			     lserver.empty()?server:lserver);
	if (!err.isOk())
	  fprintf(stderr,"%s\n",
		  err.errorString().c_str());
	j++;
      }
      i++;
    } // while
    medium.ref().unmountMedium();
  } // try
  catch (HBCI::Error lerr) {
    fprintf(stderr,"Exception: %s\n",
	    lerr.errorString().c_str());
    return HBCI::Error("importUser",err);
  }
  if (!j) {
    fprintf(stderr,"No user added.\n");
    return HBCI::Error("importUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_ABORT,
		       "no user added");
  }

  return HBCI::Error();
}



HBCI::Error createUser(HBCI::Pointer<HBCI::API> hbciif,
		       HBCI::SimpleConfig &opt){
  int country;
  string userid;
  string customerid;
  string role;
  string instid;
  string mediumname;
  string server;
  string tmp;
  HBCI::Pointer<HBCI::Bank> bank;
  HBCI::Pointer<HBCI::User> cust;
  HBCI::Pointer<HBCI::Medium> medium;
  HBCI::Error err;
  MediumType mtype;
  bool import;

  cust.setDescription("createUser::cust (HBCI::User)");

  // get some variables
  country=opt.getIntVariable("country",280,opt.root());
  instid=opt.getVariable("instid","",opt.root());
  userid=opt.getVariable("user","",opt.root());
  customerid=opt.getVariable("customer","",opt.root());
  role=opt.getVariable("role","",opt.root());
  mediumname=opt.getVariable("medium","",opt.root());
  server=opt.getVariable("server","",opt.root());
  import=opt.findVariable("import",opt.root(),false).isValid();

  if (opt.findVariable("secmode",opt.root(),false).isValid()) {
    fprintf(stderr,
	    "Option secmode is obsolete.\n"
	    "Please use \"--mediumtype\" instead.\n");
    return HBCI::Error("createUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_ABORT,
		       "secmode is obsolete");
  }

  // get the medium type
  tmp=opt.getVariable("mediumtype","file",opt.root());
  if (-1!=HBCI::parser::cmpPattern(tmp,"file",false)) {
    mtype=MediumRDHFile;
  }
  else if (-1!=HBCI::parser::cmpPattern(tmp,"card",false)) {
#ifndef HAS_LIBCHIPCARD
    return HBCI::Error("createUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_ABORT,
		       "Your version of OpenHBCI has "
		       "no support for Chipcards");
#else
    mtype=checkCardType();
#endif
  }
  else {
    return HBCI::Error("createUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_ABORT,
		       "unknown medium type");
  }


  if (mtype==MediumRDHFile) {
    if (mediumname.empty())
      return HBCI::Error("createUser()",
			 ERROR_LEVEL_NORMAL,
			 0,
			 ERROR_ADVISE_DONTKNOW,
			 "no medium path given.");
    if (mediumname.at(0)!='/') {
      // make medium path absolute
      mediumname=HBCI::Directory::workingDirectory()+"/"+mediumname;
    }
  }
  if (import)
    return importUser(hbciif,
		      mediumname,
		      mtype,
		      country,
		      instid,
		      userid,
		      customerid,
		      role,
		      server);
  return reallyCreateUser(hbciif,
			  mediumname,
			  mtype,
			  country,
			  instid,
			  userid,
			  customerid,
			  role,
			  server);
}


