/***************************************************************************
 $RCSfile: debitnote.cpp,v $
                             -------------------
    cvs         : $Id: debitnote.cpp,v 1.6 2003/02/02 22:32:50 aquamaniac Exp $
    begin       : Sun Feb 10 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 */

#include "debitnote.h"



HBCI::Error debitnote(HBCI::Pointer<HBCI::API> hbciif,
					  HBCI::SimpleConfig &opt){
    HBCI::Pointer<HBCI::OutboxJob> job;
    HBCI::cfgPtr var;
    int country;
    string instid;
    string custid;
    string accnr;
    HBCI::Error err1, err2, err3;
    string tfile;
    list<HBCI::Transaction> tdas;
    list<HBCI::Transaction> notTaken;
    list<HBCI::Transaction> taken;
    list<HBCI::Transaction>::iterator titer;
    HBCI::Pointer<HBCI::Customer> cust;
    HBCI::Pointer<HBCI::Loader> loader;
    HBCI::Pointer<HBCI::Account> acc;
    list<HBCI::Pointer<HBCI::OutboxJob> > jobs;
    list<HBCI::Pointer<HBCI::OutboxJob> >::iterator jit;

    job.setDescription("debitnote:job (HBCI::utboxJob)");
    // FIXME: there should be a better const-correct way around this
    // hack
    loader=new HBCI::Loader(const_cast<HBCI::API*>(hbciif.ptr()));
    country=opt.getIntVariable("country",280,opt.root());
    custid=opt.getVariable("customer","",opt.root());
    tfile=opt.getVariable("tfile","",opt.root());
    if (tfile.empty())
        return HBCI::Error("debitnote()",
                       ERROR_LEVEL_NORMAL,
                       0,
                       ERROR_ADVISE_DONTKNOW,
                       "no file specified");
    err1=loader.ref().readTransactionFile(tfile, tdas);
    if (!err1.isOk())
        return err1;

    for (titer=tdas.begin();
         titer!=tdas.end();
         titer++) {
        acc=hbciif.ref().findAccount((*titer).ourCountryCode(),
                                     (*titer).ourBankCode(),
                                     (*titer).ourAccountId());
        if (!acc.isValid()) {
            fprintf(stderr,"Account not found, transaction skipped.\n");
            notTaken.push_back(*titer);
        }
        else {
	    (*titer).setTransactionCode(05);

	    // get authorized customer for this account
            if (!custid.empty()) {
                // customer given
                cust=acc.ref().bank().ref().findCustomer(custid);
                if (!cust.isValid())
                    return HBCI::Error("debitnote()",
                                     ERROR_LEVEL_NORMAL,
                                     0,
                                     ERROR_ADVISE_DONTKNOW,
                                     "customer not found");
                if (!(acc.ref().isAuthorized(cust)))
                    return HBCI::Error("debitnote()",
                                     ERROR_LEVEL_NORMAL,
                                     0,
                                     ERROR_ADVISE_DONTKNOW,
                                     "customer is not authorized");
            }
            else {
                // not given, try the first in the list
                list<HBCI::Pointer<HBCI::Customer> > custlist;

                custlist=acc.ref().authorizedCustomers();
                if (custlist.empty())
                    return HBCI::Error("debitnote()",
                                     ERROR_LEVEL_NORMAL,
                                     0,
                                     ERROR_ADVISE_DONTKNOW,
                                     "no authorized customer for this account");
                cust=custlist.front();
            }

            job=new HBCI::OutboxJobDebitNote(cust,acc,*titer);
            // enqueue job
            hbciif.ref().addJob(job);
        }
    } // for

    // execute outbox
    try {
        err1=hbciif.ref().executeQueue(true);
    }
    catch (HBCI::Error lerr) {
        err1=lerr;
    }

    // check which transactions has been performed
    jobs=hbciif.ref().queuedJobs();
    for (jit=jobs.begin();
         jit!=jobs.end();
         jit++) {
        if ((*jit).ref().result()==HBCI_JOB_RESULT_SUCCESS)
            taken.push_back(((*jit).cast<HBCI::OutboxJobDebitNote>()).ref().transaction());
        else
            notTaken.push_back(((*jit).cast<HBCI::OutboxJobDebitNote>()).ref().transaction());
    }
    // save list of transactions taken
    if (!taken.empty())
        err2=loader.ref().writeTransactionFile(tfile+".taken",
                                               taken);

    // save list of transactions not taken
    if (!notTaken.empty())
        err3=loader.ref().writeTransactionFile(tfile+".nottaken",
                                               notTaken);

    if (!err1.isOk())
        return err1;
    if (!err3.isOk())
        return err3;
    return err2;
}





