/***************************************************************************
 $RCSfile: dump.cpp,v $
                             -------------------
    cvs         : $Id: dump.cpp,v 1.6 2003/01/21 02:33:49 aquamaniac Exp $
    begin       : Sun Jan 27 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 */

#include <string>
#include "xactionmatcher.h"
#include "dump.h"

#define ROWS 20
#define COLS 4

#define SIZE_DATE    12
#define SIZE_TEXT    16
#define SIZE_NAME    30
#define SIZE_PURPOSE 30
#define SIZE_VALUE   30
#define SIZE_ALL (SIZE_DATE+SIZE_TEXT+SIZE_NAME+SIZE_VALUE+9)



HBCI::Value convertValue(const HBCI::Value &v,const string &curr) {
    if (v.getCurrency()==curr)
        return v;

    if (curr=="EUR") {
        if (v.getCurrency()=="DEM")
            return HBCI::Value(v.getValue()*1.95583,"EUR");
    }
    else if (curr=="DEM") {
        if (v.getCurrency()=="EUR")
            return HBCI::Value(v.getValue()*0.511292,"DEM");
    }

    fprintf(stderr,
            "Cannot convert currency (%s -> %s)\n",
            v.getCurrency().c_str(),
            curr.c_str());
    return HBCI::Value(0,curr);
}


bool _xactionToEntry(const HBCI::Transaction &t,
                     string *e,
                     int &height) {
    string tmp1;
    string date;
    string text;
    list<string>name;
    list<string>purpose;
    string value;
    list<string>::const_iterator iter;
    int i;

    height=0;
    // date
    date=HBCI::String::num2string(t.valutaDate().day());
    date+=".";
    date+=HBCI::String::num2string(t.valutaDate().month());
    date+=".";
    date+=HBCI::String::num2string(t.valutaDate().year());

    // text
    text=t.transactionText().substr(0,SIZE_TEXT);

    // name
    for (iter=t.otherName().begin();
         iter!=t.otherName().end();
         iter++) {
        name.push_back(*iter);
    }

    // purpose
    for (iter=t.description().begin();
         iter!=t.description().end();
         iter++) {
        purpose.push_back(*iter);
    }

    // value
    if (t.value().getValue()>=0)
        value=" ";
    value+=t.value().toReadableString();

    // now put it together
    e[0]=date;
    e[1]=text;
    i=0;
    for (iter=name.begin();
         iter!=name.end() && i<(ROWS-1);
         iter++) {
        e[2+(COLS*i)]=*iter;
        i++;
    }
    e[2+(COLS*i)]="";
    i++;
    for (iter=purpose.begin();
         iter!=purpose.end() && i<(ROWS-1);
         iter++) {
        e[2+(COLS*i)]=*iter;
        i++;
    }
    height=i;
    if (!height)
        height=1;
    e[3]=value;

    return true;
}


string _entryToTable(string *e,int height, int row) {
    string result;
    string tmp;
    int i;

    if (row & 1)
        result="<TR bgcolor=\"white\">";
    else
        result="<TR bgcolor=\"lightyellow\">";
    // date
    result+="<TD>"+e[0]+"</TD>";
    // text
    result+="<TD ALIGN=\"center\">"+e[1]+"</TD>";

    // name/purpose
    tmp.erase();
    for (i=0; i<height; i++) {
        if (i)
            tmp+="<BR>";
        tmp+=e[i*COLS+2];
    }
    result+="<TD ALIGN=\"center\">"+tmp+"</TD>";

    // value
    result+="<TD ALIGN=\"right\">"+e[3]+"</TD>";

    result+="</TR>";
    return result;
}



string dumpXactionTXT(const HBCI::Transaction &t) {
    string e[COLS*ROWS];
    int i;
    string result;
    string tmp;
    int height;

    if (!_xactionToEntry(t,e,height))
        return "";
    for (i=0; i<(height*COLS); i+=COLS) {
        // date
        tmp=e[i+0];
        tmp=tmp.substr(0,SIZE_DATE);
        if (tmp.length()<SIZE_DATE)
            tmp+=string(SIZE_DATE-tmp.length(),' ');
        result+=tmp;
        result+=" | ";
        // text
        tmp=e[i+1];
        tmp=tmp.substr(0,SIZE_TEXT);
        if (tmp.length()<SIZE_TEXT)
            tmp+=string(SIZE_TEXT-tmp.length(),' ');
        result+=tmp;
        result+=" | ";
        // name / purpose
        tmp=e[i+2];
        tmp=tmp.substr(0,SIZE_NAME);
        if (tmp.length()<SIZE_NAME)
            tmp+=string(SIZE_NAME-tmp.length(),' ');
        result+=tmp;
        result+=" | ";
        // value
        tmp=e[i+3];
        tmp=tmp.substr(0,SIZE_VALUE);
        if (tmp.length()<SIZE_VALUE)
            tmp+=string(SIZE_VALUE-tmp.length(),' ');
        result+=tmp;
        result+="\n";
    } // for
    return result;
}


string dumpStandingOrderTXT(const HBCI::StandingOrder &sto) {
    string result;
	list<string>::iterator iter;

    result+="Standing Order\n";
    result+=string(SIZE_ALL,'-')+"\n";
    result+="Job number      : ";
    result+=sto.jobIdentification()+"\n";

    result+="Recipient\n";
    result+="  Account       : ";
    result+=sto.otherAccountId()+"\n";

    result+="  Country       : ";
    result+=HBCI::String::num2string(sto.otherCountryCode())+"\n";
    result+="  Bank Code     : ";
    result+=sto.otherBankCode()+"\n";
    result+="  Name          : ";
	list<string> name = sto.otherName();
	for (iter = name.begin(); iter != name.end(); iter++)
	  result += (*iter) + "\n";
    result+="Value           : ";
    result+=sto.value().toReadableString()+"\n";

    result+="Purpose         : ";
    list<string> purpose = sto.description();
    if (purpose.empty())
        result+="\n";
    else
        for (iter = purpose.begin(); iter != purpose.end(); iter++)
            result += (*iter)+ "\n";

    result+="Textkey         : ";
    result+=sto.transactionCode();
    result+="\n";
    result+="First execution : ";
    result+=sto.firstExecutionDate().toString()+"\n";
    result+="Last execution  : ";
    result+=sto.lastExecutionDate().toString()+"\n";
    result+="Next execution  : ";
    result+=sto.executionDate().toString()+"\n";
    result+="cycle           : every ";
    if (sto.cycle()>1)
        result+=HBCI::String::num2string(sto.cycle())+" ";
    if (sto.period()==HBCI::StandingOrder::EXECUTE_MONTHLY) {
        result+="month";
        if (sto.cycle()>1)
            result+="s";
    }
    else {
        result+="week";
        if (sto.cycle()>1)
            result+="s";
    }
    result+="\n";

    return result;
}


string dumpStandingOrderHTML(const HBCI::StandingOrder &sto) {
    string result;
	list<string>::iterator iter;

    result+="<h5>Standing Order</h5>\n";
    result+="<TABLE BORDER=1>";
    result+="<tr><td>Job number</td><td>";
    result+=sto.jobIdentification()+"</td></tr>\n";

    result+="<tr><td>Recipient Account</td><td>\n";
    result+=sto.otherAccountId()+"</td></tr>\n";

    result+="<tr><td>Recipient Country</td><td>";
    result+=HBCI::String::num2string(sto.otherCountryCode())+"</td></tr>\n";
    result+="<tr><td>Recipient Bank Code</td><td>";
    result+=sto.otherBankCode()+"</td></tr>\n";
    result+="<tr><td>Recipient Name</td><td>";
	list<string> name = sto.otherName();
	for (iter = name.begin(); iter != name.end(); iter++)
	  result += (*iter) + "<br>\n";
    result+="</td></tr>";
    result+="<tr><td>Value</td><td>";
    result+=sto.value().toReadableString()+"</td></tr>\n";

    result+="<tr><td>Purpose</td><td>";
	list<string> purpose = sto.description();
	for (iter = purpose.begin(); iter != purpose.end(); iter++)
	  result += (*iter)+ "<br>\n";
	result += "</td></tr>\n";

    result+="<tr><td>Textkeys</td><td>";
    result+=sto.transactionCode()+"</td></tr>\n";
    result+="<tr><td>First execution</td><td>";
    result+=sto.firstExecutionDate().toString()+"</td></tr>\n";
    result+="<tr><td>Last execution</td><td>";
    result+=sto.lastExecutionDate().toString()+"</td></tr>\n";
    result+="<tr><td>Next execution</td><td>";
    result+=sto.executionDate().toString()+"</td></tr>\n";
    result+="<tr><td>cycle</td><td>every ";
    if (sto.cycle()>1)
        result+=HBCI::String::num2string(sto.cycle())+" ";
    if (sto.period()==HBCI::StandingOrder::EXECUTE_MONTHLY) {
        result+="month";
        if (sto.cycle()>1)
            result+="s";
    }
    else {
        result+="week";
        if (sto.cycle()>1)
            result+="s";
    }
    result+="</td></tr>\n";

    result+="</table><br>";
    return result;
}


string dumpBankMessageTXT(const HBCI::instituteMessage &m) {
    string result;

    result+="Institute Message\n";
    result+=string(SIZE_ALL,'-')+"\n";
    result+="Date    : ";
    result+=m.date().toString()+"\n";
    result+="Time    : ";
    result+=m.time().toString()+"\n";
    result+="Subject : ";
    result+=m.subject()+"\n";
    result+="Text    : ";
    result+=m.text()+"\n";

    result+="\n";
    return result;
}


string dumpBankMessageHTML(const HBCI::instituteMessage &m) {
    string result;

    result+="<h5>Institute Message</h5>\n";
    result+="<table>";
    result+="<tr><td>";
    result+="<b>Date</b>";
    result+="</td><td>";
    result+=m.date().toString()+"\n";
    result+="</td></tr>";
    result+="<tr><td>";
    result+="<b>Time</b> ";
    result+="</td><td>";
    result+=m.time().toString()+"\n";
    result+="</td></tr>";
    result+="<tr><td>";
    result+="<b>Subject</b> ";
    result+="</td><td>";
    result+=m.subject()+"\n";
    result+="</td></tr>";
    result+="<tr><td>";
    result+="<b>Text</b> ";
    result+="</td><td>";
    result+=m.text()+"\n";
    result+="</td></tr>";
    result+="</table>";
    result+="<br>\n";

    return result;
}


string dumpBalanceTXT(const HBCI::AccountBalance &b, 
		      const HBCI::Value &vin,
		      const HBCI::Value &vout) {
    string result;
    HBCI::Value tmpval;

    if (vin.getValue() || vout.getValue()) {
        result+="In: ";
        result+=vin.toReadableString()+"  ";
        result+="Out: ";
        result+=vout.toReadableString()+"  ";
        result+="Diff: ";
        tmpval=HBCI::Value(vin.getValue()-vout.getValue(),vin.getCurrency());
        result+=tmpval.toReadableString()+" ";
        result+="\n";
    }
    if (!b.bookedBalance().date().isValid()) {
        result+=" --- no noted balance ---";
    }
    else {
        result+="Noted balance since ";
        result+=HBCI::String::num2string(b.bookedBalance().date().day());
        result+=".";
        result+=HBCI::String::num2string(b.bookedBalance().date().month());
        result+=".";
        result+=HBCI::String::num2string(b.bookedBalance().date().year());
        result+=" at ";
        result+=HBCI::String::num2string(b.bookedBalance().time().hour(),true,2);
        result+=":";
        result+=HBCI::String::num2string(b.bookedBalance().time().minute(),true,2);
        result+=":";
        result+=HBCI::String::num2string(b.bookedBalance().time().second(),true,2);
        result+=" : ";
        result+=(b.bookedBalance().isDebit())?"-":"+";
        result+=b.bookedBalance().value().toReadableString();
        result+=" ";
        result+=b.currency();
    }
    result+="\n";
    return result;
}


string dumpBalanceHTML(const HBCI::AccountBalance &b,
		       const HBCI::Value &vin,
		       const HBCI::Value &vout) {
    string result;
    HBCI::Value tmpval;

    result+="<h4>Balance</h4>";
    if (vin.getValue() || vout.getValue()) {
        result+="<table><tr>";
        result+="<td><b>In</b></td><td> ";
        result+=vin.toReadableString()+"</td> ";
        result+="<td><b>Out</b></td><td>";
        result+=vout.toReadableString()+"</td>";
        result+="<td><b>Diff</b></td><td> ";
        if (vout<vin)
            result+="<font color=green>";
        else
            result+="<font color=red>";
        tmpval=HBCI::Value(vin.getValue()-vout.getValue(),vin.getCurrency());
        result+=tmpval.toReadableString()+"</font></td></tr>";
        result+="</table>\n";
    }
    if (!b.bookedBalance().date().isValid()) {
        result+="<font color=red> --- no noted balance ---</font>";
    }
    else {
        result+="Noted balance since ";
        result+=HBCI::String::num2string(b.bookedBalance().date().day());
        result+=".";
        result+=HBCI::String::num2string(b.bookedBalance().date().month());
        result+=".";
        result+=HBCI::String::num2string(b.bookedBalance().date().year());
        result+=" at ";
        result+=HBCI::String::num2string(b.bookedBalance().time().hour(),true,2);
        result+=":";
        result+=HBCI::String::num2string(b.bookedBalance().time().minute(),true,2);
        result+=":";
        result+=HBCI::String::num2string(b.bookedBalance().time().second(),true,2);
        result+=" : ";
        result+=(b.bookedBalance().isDebit())?"-":"+";
        if (b.bookedBalance().isDebit())
            result+="<font color=red>";
        else
            result+="<font color=green>";
        result+=b.bookedBalance().value().toReadableString();
        result+="</font> ";
        result+=b.currency();
    }
    result+="<br>";
    return result;
}


string dumpAccountTXT(HBCI::Pointer<HBCI::API> hbciif,
		      HBCI::Pointer<HBCI::Account> acc,
		      unsigned int flags,
		      const HBCI::Date &fromdate,
		      const HBCI::Date &todate){
  string result;
  string tmp;
  HBCI::Pointer<HBCI::Bank> bank;
  HBCI::Balance bal;
  double vin=0.0;
  double vout=0.0;
  HBCI::Value tmpval;
  list<HBCI::StandingOrder>::const_iterator stit;
  TransactionMatcher tm;

  result+="Konto ";
  result+=acc.ref().accountId();
  // get institute name
  bank=acc.ref().bank();

  result+=" bei ";
  result+=bank.ref().name();

  result+=" (BLZ: ";
  result+=bank.ref().bankCode();
  result+=")\n";
  tm=TransactionMatcher(0,
			"*",
			"*",
			fromdate,
			todate,
			flags & 0xffff);

  if (flags & AQM_DUMP_FLAGS_XACTIONS) {
    if (acc.ref().transactions().empty())
      result+=" --- no transactions ---\n";
    else {
      result+="Transactions\n";
      result+=string(SIZE_ALL,'=')+"\n";
      for (list<HBCI::Transaction>::const_iterator i2=acc.ref().transactions().begin();
	   i2!=acc.ref().transactions().end(); i2++) {
	if (!tm.match(*i2))
	  continue;
	// dump
	result+=dumpXactionTXT(*i2);
	result+=string(SIZE_ALL,'-');

	// compute values
	tmpval=convertValue((*i2).value(),"EUR");
	if (tmpval.getValue()<0)
	  vout+=-(tmpval.getValue());
	else
	  vin+=tmpval.getValue();

	result+="\n";
      } // for
    } // if there are xactions
  } // if wantXactions

  // show balance
  if (flags & AQM_DUMP_FLAGS_BALANCE) {
    result+="\nBalance\n";
    result+=string(SIZE_ALL,'=')+"\n";
    result+=dumpBalanceTXT(acc.ref().balance(),
			   HBCI::Value(vin,acc.ref().currency()),
			   HBCI::Value(vout,acc.ref().currency()));
  }

  // show standing orders
  if (flags & AQM_DUMP_FLAGS_STO) {
    result+="\nList of Standing Orders\n";
    result+=string(SIZE_ALL,'=')+"\n";
    for (stit=acc.ref().standingOrders().begin();
	 stit!=acc.ref().standingOrders().end();
	 stit++)
      result+=dumpStandingOrderTXT(*stit);
  }

  result+="\n\n";

  return result;
}


string dumpAccountHTML(HBCI::Pointer<HBCI::API> hbciif,
                       HBCI::Pointer<HBCI::Account> acc,
		       unsigned int flags,
                       const HBCI::Date &fromdate,
                       const HBCI::Date &todate){
    list<HBCI::Transaction>::const_iterator iter;
    list<HBCI::StandingOrder>::const_iterator stit;
    string e[COLS*ROWS];
    int height;
    string result;
    string name;
    string descr;
    int row;
    HBCI::Pointer<HBCI::Bank> bank;
    double vin=0.0;
    double vout=0.0;
    HBCI::Value tmpval;
    TransactionMatcher tm;

    result+="<H2>Account <i>";
    if (acc.ref().accountName().empty())
        result+=acc.ref().accountId();
    else
        result+=acc.ref().accountName();
    result+="</i> at <i>";

    // get institute name
    bank=acc.ref().bank();

    result+=bank.ref().name();
    result+="</i></H2><BR>\n";
    tm=TransactionMatcher(0,
			  "*",
			  "*",
			  fromdate,
			  todate,
			  flags & 0xffff);

    if (flags & AQM_DUMP_FLAGS_XACTIONS) {
      if (acc.ref().transactions().empty())
	result+=" --- no transactions ---<BR>\n";
      else {
	result+="<h4>Transactions</h4>\n";
	result+="<TABLE BORDER=\"1\" WIDTH=\"100%\">\n";
	result+="<TR>";
	result+="<TH>Date</TH>";
	result+="<TH>Text</TH>";
	result+="<TH>Name/Purpose</TH>";
	result+="<TH>Value</TH>";
	result+="</TR>\n";

	row=0;
	for (iter=acc.ref().transactions().begin();
	     iter!=acc.ref().transactions().end();
	     iter++) {
	  // match transaction
	  if (!tm.match(*iter))
	    continue;

	  // dump
	  height=0;
	  if (!_xactionToEntry((*iter),e,height))
	    return "";
	  result+=_entryToTable(e,height,row);
	  result+="\n";

	  // compute values
	  tmpval=convertValue((*iter).value(),"EUR");
	  if (tmpval.getValue()<0)
	    vout+=-(tmpval.getValue());
	  else
	    vin+=tmpval.getValue();

	  row++;
	} // for
	result+="</TABLE>\n";
      }
    } // if wantXActions
    if (flags & AQM_DUMP_FLAGS_BALANCE)
      result+=dumpBalanceHTML(acc.ref().balance(),
			      HBCI::Value(vin,acc.ref().currency()),
			      HBCI::Value(vout,acc.ref().currency()));
    // show standing orders
    if (flags & AQM_DUMP_FLAGS_STO) {
      result+="<h4>List of Standing Orders</h4>\n";
      for (stit=acc.ref().standingOrders().begin();
	   stit!=acc.ref().standingOrders().end();
	   stit++)
	result+=dumpStandingOrderHTML(*stit);
    }
    result+="<BR><BR>\n";
    return result;
}


HBCI::Error dump(HBCI::Pointer<HBCI::API> hbciif,
		 HBCI::SimpleConfig &opt){
  HBCI::Tree<HBCI::ConfigNode>::iterator var;
  int country;
  string instid;
  string accnr;
  list<HBCI::Pointer<HBCI::Account> > accounts;
  list<HBCI::Pointer<HBCI::Account> >::const_iterator iter;
  HBCI::Error err;
  unsigned int flags=0;
  string result;
  string tmp;
  string fdate;
  string tdate;
  HBCI::Date fromdate;
  HBCI::Date todate;
  list<HBCI::Pointer<HBCI::Bank> > banks;
  list<HBCI::Pointer<HBCI::Bank> >::const_iterator bit;
  list<HBCI::instituteMessage>::const_iterator mit;

  country=opt.getIntVariable("country",0,opt.root());
  instid=opt.getVariable("instid","*",opt.root());
  accnr=opt.getVariable("accnr","*",opt.root());
  if (opt.findVariable("balance",opt.root(),false).isValid())
    flags|=AQM_DUMP_FLAGS_BALANCE;
  if (opt.findVariable("xactions",opt.root(),false).isValid())
    flags|=AQM_DUMP_FLAGS_XACTIONS;

  fdate=opt.getVariable("fromdate","",opt.root());
  tdate=opt.getVariable("todate","",opt.root());
  tmp=opt.getVariable("output","txt",opt.root());
  if (-1!=HBCI::parser::cmpPattern(tmp,
				   "html",
				   false))
    flags|=AQM_DUMP_FLAGS_HTML;

  if (opt.findVariable("sto",opt.root(),false).isValid())
    flags|=AQM_DUMP_FLAGS_STO;
  if (opt.findVariable("msg",opt.root(),false).isValid())
    flags|=AQM_DUMP_FLAGS_MESSAGE;

  flags|=TM_FLAGS_POSITIVE;
  flags|=TM_FLAGS_NEGATIVE;
  if (opt.findVariable("pos",opt.root(),false).isValid())
    flags&=~TM_FLAGS_NEGATIVE;
  if (opt.findVariable("neg",opt.root(),false).isValid())
    flags&=~TM_FLAGS_POSITIVE;

  // set from date
  if (!fdate.empty()) {
    fromdate=HBCI::Date(fdate,4);
    if (!fromdate.isValid())
      return HBCI::Error("turnover()",
			 ERROR_LEVEL_NORMAL,
			 0,
			 ERROR_ADVISE_DONTKNOW,
			 "error in date format.");
  }
  // set todate
  if (!tdate.empty()) {
    todate=HBCI::Date(tdate,4);
    if (!todate.isValid())
      return HBCI::Error("turnover()",
			 ERROR_LEVEL_NORMAL,
			 0,
			 ERROR_ADVISE_DONTKNOW,
			 "error in date format.");
  }

  // get accounts
  accounts=hbciif.ref().getAccounts(country,instid,accnr);

  // create head
  if (flags & AQM_DUMP_FLAGS_HTML) {
    result+="<!DOCTYPE HTML PUBLIC \"";
    result+="-//W3C//DTD HTML 4.01 Transitional//EN\">";
    result+="<HTML>";
    result+="<meta name=\"Generator\" content=\"AqMoney 0.1\">\n";
    result+="<meta name=\"Author\" content=\"Martin Preuss\">\n";
    result+="<meta http-equiv=\"Content-Type\"";
    result+="content=\"text/html; charset=ISO-8859-1\">\n";
    result+="<title>Transactions</title>\n";
    result+="</head>\n";
    result+="<BODY>\n";
  }
  else
    result.erase();
  for (iter=accounts.begin();
       iter!=accounts.end();
       iter++) {
    if (flags & AQM_DUMP_FLAGS_HTML)
      result+=dumpAccountHTML(hbciif,
			      *iter,
                              flags,
			      fromdate,
			      todate)+"<br>\n";
    else
      result+=dumpAccountTXT(hbciif,
			     *iter,
                             flags,
			     fromdate,
			     todate)+"\n";
  } // for

  // show institute messages
  if (flags & AQM_DUMP_FLAGS_MESSAGE) {
    banks=hbciif.ref().bankList();
    if (flags & AQM_DUMP_FLAGS_HTML)
      result+="<h4>Institute Messages</h4>";
    else {
      result+="Institute Messages\n";
      result+=string(SIZE_ALL,'=')+"\n";
    }
    for (bit=banks.begin();
	 bit!=banks.end();
	 bit++) {
      if ((*bit).ref().messages().empty())
	break;

      if (flags & AQM_DUMP_FLAGS_HTML) {
	result+="<h5>Messages from ";
	result+=(*bit).ref().name();
	result+=" (BLZ: ";
	result+=(*bit).ref().bankCode();
	result+=")</h5>\n";
      }
      else {
	result+="\nMessages from ";
	result+=(*bit).ref().name();
	result+=" (BLZ: ";
	result+=(*bit).ref().bankCode();
	result+=")\n\n";
      }
      for (mit=(*bit).ref().messages().begin();
	   mit!=(*bit).ref().messages().end();
	   mit++) {
	if (flags & AQM_DUMP_FLAGS_HTML)
	  result+=dumpBankMessageHTML(*mit);
	else
	  result+=dumpBankMessageTXT(*mit);
      } // for message
    } // for bank
  } // if wantMSG

  if (flags & AQM_DUMP_FLAGS_HTML)
    result+="</BODY></HTML>";

  // print result
  fprintf(stdout,"%s",result.c_str());
  return HBCI::Error();
}





