/***************************************************************************
 $RCSfile: export.cpp,v $
                             -------------------
    cvs         : $Id: export.cpp,v 1.5 2003/01/21 02:33:49 aquamaniac Exp $
    begin       : Mon May 06 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 */



#include "export.h"
#include "xactionmatcher.h"


string _qhacc_dump(const HBCI::Transaction &t, int i) {
    string result;
    list<string>::const_iterator iter;
    HBCI::Value val;
    float fval;
    bool isneg;
    HBCI::Date d;

    // serial num ? for now we use the number
    result+=t.ourAccountId();
    result+="-";
    result+=HBCI::String::num2string(i);
    result+="||";

    // dont know
    result+="0||";

    // num
    result+=HBCI::String::num2string(i);
    result+="||";

    // name
    for (iter=t.otherName().begin();
         iter!=t.otherName().end();
         iter++) {
        if (iter!=t.otherName().begin())
            result+=" ";
        result+=(*iter);
    }
    result+="||";

    // purpose
    for (iter=t.description().begin();
         iter!=t.description().end();
         iter++) {
        if (iter!=t.description().begin())
            result+=" ";
        result+=(*iter);
    }
    result+="||";

    // value
    fval=t.value().getValue();
    isneg=(fval<0);
    val=HBCI::Value(fval,"");
    result+=val.toReadableString();
    result+="||";

    // direction
    if (isneg)
        result+="OUT";
    else
        result+="IN";
    result+="||";

    // date
    if (t.valutaDate().isValid())
        d=t.valutaDate();
    else
        d=t.date();
    result+=HBCI::String::num2string(d.month());
    result+="/";
    result+=HBCI::String::num2string(d.day());
    result+="/";
    result+=HBCI::String::num2string(d.year());

    // that's it
    return result;
}


HBCI::Error export_qhacc(HBCI::Pointer<HBCI::Account> acc,
                       string fname) {
    HBCI::Error err;
    HBCI::Error err2;
    string result;
    int i;
    HBCI::Pointer<HBCI::File> file;
    list<HBCI::Transaction>::const_iterator iter;

    // store accounts
    i=1;
    result+="NUMTRANS=";
    result+=HBCI::String::num2string(acc.ref().transactions().size());
    result+="\n";
    for (iter=acc.ref().transactions().begin();
         iter!=acc.ref().transactions().end(); iter++) {
        // dump transaction to file
        result+=_qhacc_dump((*iter),i);
        result+="\n";
        i++;
    } // for transaction data

    // write to file
    file=new HBCI::File(fname);
    err=file.ref().createFile(FILE_AM_RDWR|FILE_AM_CREATE_ALWAYS);
    if (!err.isOk())
        return err;
    err=file.ref().writeData(result);
    err2=file.ref().closeFile();
    if (!err.isOk())
        return err;
    if (!err2.isOk())
        return err2;
    return HBCI::Error();
}


string _txt_dump(const HBCI::Transaction &t, unsigned int flags) {
  string result;
  list<string>::const_iterator iter;
  HBCI::Date d;
  HBCI::Value val;
  float fval;

  // name
  result+="\"";
  for (iter=t.otherName().begin();
       iter!=t.otherName().end();
       iter++) {
    if (iter!=t.otherName().begin())
      result+=" ";
    result+=(*iter);
  }
  result+="\"";
  result+=";";

  // purpose
  result+="\"";
  for (iter=t.description().begin();
       iter!=t.description().end();
       iter++) {
    if (iter!=t.description().begin())
      result+=" ";
    result+=(*iter);
  }
  result+="\"";
  result+=";";

  // value
  result+="\"";
  fval=t.value().getValue();
  val=HBCI::Value(fval,"");
  result+=val.toReadableString();
  result+="\"";
  result+=";";

  // currency
  result+="\"";
  result+=t.value().getCurrency();
  result+="\"";
  result+=";";

  // date
  result+="\"";
  if (t.valutaDate().isValid())
    d=t.valutaDate();
  else
    d=t.date();
  result+=HBCI::String::num2string(d.day());
  result+=".";
  result+=HBCI::String::num2string(d.month());
  result+=".";
  result+=HBCI::String::num2string(d.year());
  result+="\"";
  result+=";";

  // that's it
  return result;
}


HBCI::Error export_txt(HBCI::Pointer<HBCI::Account> acc,
		       const string &fname,
		       unsigned int flags,
		       const HBCI::Date &fromdate,
		       const HBCI::Date &todate){
  HBCI::Error err;
  HBCI::Error err2;
  string result;
  int i;
  HBCI::Pointer<HBCI::File> file;
  list<HBCI::Transaction>::const_iterator iter;
  TransactionMatcher tm;

  tm=TransactionMatcher(0,
			"*",
			"*",
			fromdate,
			todate,
			flags & 0xffff);

  // store accounts
  result+="\"Name\";";
  result+="\"Purpose\";";
  result+="\"Value\";";
  result+="\"Currency\";";
  result+="\"Date\"\n";

  for (iter=acc.ref().transactions().begin();
       iter!=acc.ref().transactions().end(); iter++) {
    // dump transaction to file
    if (!tm.match(*iter))
      continue;
    result+=_txt_dump((*iter), flags);
    result+="\n";
    i++;
  } // for transaction data

  // write to file
  file=new HBCI::File(fname);
  err=file.ref().createFile(FILE_AM_RDWR|FILE_AM_CREATE_ALWAYS);
  if (!err.isOk())
    return err;
  err=file.ref().writeData(result);
  err2=file.ref().closeFile();
  if (!err.isOk())
    return err;
  if (!err2.isOk())
    return err2;
  return HBCI::Error();
}




HBCI::Error texport(HBCI::Pointer<HBCI::API> hbciif,
		    HBCI::SimpleConfig &opt){
  HBCI::cfgPtr var;
  int country;
  string instid;
  string accnr;
  list<HBCI::Pointer<HBCI::Account> > acclist;
  list<HBCI::Pointer<HBCI::Account> >::const_iterator it;
  HBCI::Error err;
  HBCI::Error err2;
  string result;
  string fname;
  string tmp;
  string format;
  unsigned int flags=0;
  HBCI::Date fromdate;
  HBCI::Date todate;

  country=opt.getIntVariable("country",0,opt.root());
  instid=opt.getVariable("instid","*",opt.root());
  accnr=opt.getVariable("accnr","*",opt.root());
  fname=opt.getVariable("outfile","",opt.root());
  format=opt.getVariable("outformat","",opt.root());
  fromdate=opt.getVariable("fromdate","",opt.root());
  todate=opt.getVariable("todate","",opt.root());

  flags|=TM_FLAGS_POSITIVE;
  flags|=TM_FLAGS_NEGATIVE;
  if (opt.findVariable("pos",opt.root(),false).isValid())
    flags&=~TM_FLAGS_NEGATIVE;
  if (opt.findVariable("neg",opt.root(),false).isValid())
    flags&=~TM_FLAGS_POSITIVE;

  if (fname.empty())
    return HBCI::Error("texport()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_DONTKNOW,
		       "name of output file needed.");
  if (format.empty())
    return HBCI::Error("texport()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_DONTKNOW,
		       "name of output format needed.");

  // get accounts
  acclist=hbciif.ref().getAccounts(country,instid,accnr);
  if (acclist.empty())
    return HBCI::Error("texport()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_DONTKNOW,
		       "no account found.");

  // if no data then simply return
  for (it=acclist.begin();
       it!=acclist.end();
       it++) {
    if (!(*it).ref().transactions().empty()) {

      // output
      if (format=="qhacc")
	err=export_qhacc(*it,fname);
      else if (format=="txt")
	err=export_txt(*it,fname,
		       flags,
		       fromdate,
		       todate);
      else
	err=HBCI::Error("texport()",
			ERROR_LEVEL_NORMAL,
			0,
			ERROR_ADVISE_DONTKNOW,
			"unknown output format.");
      if (!err.isOk())
	return err;
    } // if transactions
  } // for

  return HBCI::Error();
}





