/***************************************************************************
 $RCSfile: main.cpp,v $
                             -------------------
    cvs         : $Id: main.cpp,v 1.21 2003/03/30 01:56:35 aquamaniac Exp $
    begin       : Sat Jan 26 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


// general includes

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <openhbci.h>

#include "aqminteractor.h"
// the modules of aqmoney
#include "getbalance.h"
#include "createuser.h"
#include "getkeys.h"
#include "sendkeys.h"
#include "iniletter.h"
#include "turnover.h"
#include "acclist.h"
#include "dump.h"
#include "accadd.h"
#include "transfer.h"
#include "debitnote.h"
#include "sync.h"
#include "export.h"
#include "getstandingorders.h"
#include "newstandingorder.h"
#include "delstandingorder.h"
#include "createcustomer.h"
#include "resetseq.h"
#include "report.h"
#include "disablekeys.h"
#include "changekeys.h"
#include "modifymedium.h"
#include "chgversion.h"


s_CmdLineOptionDescr options[]={
{
  "user",                                    // config item name
  CLO_FLAGS_HAS_ARGUMENT,                    // flags
  "user",                                    // long option name
  "u",                                       // short option name
  "userid assigned to you by your institute" // description
},
{
  "customer",                                // config item name
  CLO_FLAGS_HAS_ARGUMENT,                    // flags
  "customer",                                // long option name
  "",                                        // short option name
  "customerid assigned to you by your institute" // description
},
{
  "role",                                    // config item name
  CLO_FLAGS_HAS_ARGUMENT,                    // flags
  "role",                                    // long option name
  "",                                        // short option name
  "role of the customer (e.g. \"private\")"  // description
},
{
  "mediumtype",                              // config item name
  CLO_FLAGS_HAS_ARGUMENT,                    // flags
  "mediumtype",                              // long option name
  "",                                        // short option name
  "either \"file\" or \"card\""              // description
},
{
  "instid",
  CLO_FLAGS_HAS_ARGUMENT,
  "institute",
  "i",
  "code of your credit institute (\"Bankleitzahl\")"
},
{
  "country",
  CLO_FLAGS_HAS_ARGUMENT,
  "country",
  "c",
  "country code (\"280\" for Germany)"
},
{
  "hversion",
  CLO_FLAGS_HAS_ARGUMENT,
  "hversion",
  "",
  "HBCI version to set by command \"chgversion\""
},
{
  "medium",
  CLO_FLAGS_HAS_ARGUMENT,
  "medium",
  "",
  "DDV: cardnumber, RDH: path to the file to create"
},
{
  "secmode", // will fade ! Please use mediumtype instead
  CLO_FLAGS_HAS_ARGUMENT,
  "security",
  "",
  "security mode (DDV or RDH). Deprecated, use \"mediumtype\" instead."
},
{
  "server",
  CLO_FLAGS_HAS_ARGUMENT,
  "server",
  "",
  "ip address of the institutes server"
},
{
  "key",
  CLO_FLAGS_HAS_ARGUMENT,
  "key",
  "",
  "key to select (\"user\" OR \"institute\")"
},
{
  "output",
  CLO_FLAGS_HAS_ARGUMENT,
  "output",
  "o",
  "what to output (txt OR html"
},
{
  "accnr",
  CLO_FLAGS_HAS_ARGUMENT,
  "account",
  "a",
  "account number"
},
{
  "cmd",
  CLO_FLAGS_HAS_ARGUMENT,
  "command",
  "d",
  "what to do (see man)"
},
{
  "debuglevel",
  CLO_FLAGS_HAS_ARGUMENT,
  "debuglevel",
  "",
  "Debug level (the higher the level the more output you'll see)"
},
{
  "ro",
  0,
  "readonly",
  "r",
  "does not allow alteration of your account's balance"
},
{
  "import",
  0,
  "import",
  "",
  "import users from an existing medium"
},
{
  "cfgfile",
  CLO_FLAGS_HAS_ARGUMENT,
  "configfile",
  "C",
  "use this config file"
},
{
  "fromdate",
  CLO_FLAGS_HAS_ARGUMENT,
  "fromdate",
  "",
  "first date (YYYYMMDD)"
},
{
  "todate",
  CLO_FLAGS_HAS_ARGUMENT,
  "todate",
  "",
  "last date (YYYYMMDD)"
},
{
  "tfile",
  CLO_FLAGS_HAS_ARGUMENT,
  "tfile",
  "",
  "name of file containing transfers to be done"
},
{
  "outfile",
  CLO_FLAGS_HAS_ARGUMENT,
  "outfile",
  "",
  "name of file to write to"
},
{
  "outformat",
  CLO_FLAGS_HAS_ARGUMENT,
  "outformat",
  "",
  "format of output file"
},
{
  "xactions",
  0,
  "transactions",
  "xa",
  "used with \"dump\" to show transactions"
},
{
  "balance",
  0,
  "balance",
  "",
  "used with \"dump\" to show balances"
},
{
  "sto",
  0,
  "sto",
  "",
  "used with \"dump\" to show standing orders"
},
{
  "pos",
  0,
  "pos",
  "",
  "used with \"dump\" to show only positive values"
},
{
  "neg",
  0,
  "neg",
  "",
  "used with \"dump\" to show only negative values"
},
{
  "msg",
  0,
  "msg",
  "",
  "used with \"dump\" to show institute messages"
},
{
  "syncmode",
  CLO_FLAGS_HAS_ARGUMENT,
  "syncmode",
  "",
  "select the sync mode (for command sync)"
},
{
  "year",
  CLO_FLAGS_HAS_ARGUMENT,
  "year",
  "",
  "choose the year (for command report)"
},
{
  "month",
  CLO_FLAGS_HAS_ARGUMENT,
  "month",
  "",
  "choose the month (for command report)"
},
{
  "rflags",
  CLO_FLAGS_HAS_ARGUMENT,
  "rflags",
  "",
  "control the output of the command report (defaults to \"ymd\")"
},
{
  "keynumber",
  CLO_FLAGS_HAS_ARGUMENT,
  "keynumber",
  "",
  "number of the key to disable (used with \"disablekeys\" if you do not have access to your keys anymore)"
},
{
  "keyversion",
  CLO_FLAGS_HAS_ARGUMENT,
  "keyversion",
  "",
  "version of the key to disable (used with \"disablekeys\" if you do not have access to your keys anymore)"
},
{
  "newuser",
  CLO_FLAGS_HAS_ARGUMENT,
  "newuser",
  "",
  "store a new user id on the medium (used with \"modifymedium\")"
},
{
  "newcustomer",
  CLO_FLAGS_HAS_ARGUMENT,
  "newcustomer",
  "",
  "store a new customer id on the medium (used with \"modifymedium\")"
},
{
  "newinstitute",
  CLO_FLAGS_HAS_ARGUMENT,
  "newinstitute",
  "",
  "store a new institute id (german: BLZ) on the medium (used with \"modifymedium\")"
},
{
  "newserver",
  CLO_FLAGS_HAS_ARGUMENT,
  "newserver",
  "",
  "store a new server address on the medium (used with \"modifymedium\")"
},
{
  "newpin",
  0,
  "newpin",
  "",
  "store a new server address on the medium (used with \"modifymedium\")"
},
{
  "resetsystemid",
  0,
  "resetsystemid",
  "",
  "reset the system id (only with rdh mediums, used with \"modifymedium\"). Call -d sync afterwards"
},
{
  "help",
  CLO_FLAGS_LAST,
  "help",
  "h",
  "show this help screen"
}
};



/**
 * Main routine.
 * Return: 0 on succes, otherwise:
 *         Bit 0=1: Error in command line arguments
 *         Bit 1=1: Error loading configuration file
 *         Bit 2=1: Error saving configuration file
 *         Bit 3=1: Error on command execution
 */
int _main(int argc, char **argv) {
  HBCI::Error err;
  HBCI::CmdLineOptions opt;
  HBCI::Tree<HBCI::ConfigNode>::iterator var;
  HBCI::Pointer<HBCI::API> hbciif;
  HBCI::Pointer<AQMInteractor> interactor;
  bool ro;
  string cmd;
  string cfgfile;
  int rv;
  HBCI::Pointer<HBCI::File> f;
  int debugLevel;

  hbciif.setDescription("main::hbciif (HBCI::API)");
  err=opt.parseOptions(argc,argv,options);
  if (!err.isOk()) {
    fprintf(stderr,"Encountered an error: %s\n",
	    err.message().c_str());
    return 1;
  }
  // check if help requested
  var=opt.findVariable("help",
		       opt.root(),
		       false);
  if (var.isValid()) {
    fprintf(stdout,"Usage:\n");
    fprintf(stdout," %s <OPTIONS>\n",argv[0]);
    fprintf(stdout," where <OPTIONS> can be:\n");
    fprintf(stdout,"%s\n",opt.usage(options).c_str());
    return 1;
  }

  // get some variables
  var=opt.findVariable("ro",opt.root(),false);
  ro=var.isValid();
  cmd=opt.getVariable("cmd","",opt.root());
  if (cmd.empty()) {
    fprintf(stderr,"ERROR: no command given.\n");
    return 1;
  }
  debugLevel=opt.getIntVariable("debuglevel",0,opt.root());

  // create hbci interface
  hbciif=new HBCI::API(ro);
  hbciif.ref().setDebugLevel(debugLevel);
  hbciif.ref().setSystemName("AqMoney");
  hbciif.ref().setSystemVersion(k_AQMONEY_VERSION_STRING);
  interactor=new AQMInteractor();
  hbciif.ref().setInteractor(interactor.cast<HBCI::Interactor>());
  /* only read configuration file if it exists. Otherwise it will be
   * created in the end */

  // check whether the name of the configuration file is given
  cfgfile=opt.getVariable("cfgfile","",
			  opt.root());
  if (cfgfile.empty()) {
    /* no name given, so try some
     * We will look for these files:
     * ~/.openhbci
     * ~/aqmoney2.conf
     * If none is found then a new file will be created upon exit using
     * the first name in the list above.
     */
    cfgfile=HBCI::Directory::homeDirectory()+"/.openhbci";
    f=new HBCI::File(cfgfile);
    if (!(f.ref().accessFile(FILE_ACCESS_EXIST).isOk())) {
      // try another one
      cfgfile=HBCI::Directory::homeDirectory()+"/aqmoney2.conf";
      if (!(f.ref().accessFile(FILE_ACCESS_EXIST).isOk())) {
	// use default configuration file
	cfgfile=HBCI::Directory::homeDirectory()+"/.openhbci";
      }
    }
  }

  f=new HBCI::File(cfgfile);
  if (f.ref().accessFile(FILE_ACCESS_EXIST).isOk()) {
    err=hbciif.ref().loadEnvironment(cfgfile);

    if (!err.isOk()) {
      fprintf(stderr,"ERROR: %s\n",
	      err.errorString().c_str());
      return 0x02;
    }
  }
  else {
    fprintf(stderr,
	    "Warning: Configuration file not found, will create\n"
	    "\"%s\" later.", cfgfile.c_str());
  }

  rv=0x08;

  try {
    // do some nasty things
    if (-1!=HBCI::parser::cmpPattern(cmd,"balance",false)) {
      // get balance
      err=getBalance(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"turnover",false)) {
      err=turnover(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"createuser",false)) {
      err=createUser(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"createcustomer",false)) {
      err=createCustomer(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"acclist",false)) {
      err=acclist(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"getkeys",false)){
      err=getKeys(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"dump",false)) {
      err=dump(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"sendkeys",false)) {
      err=sendKeys(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"iniletter",false)) {
      err=iniletter(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"transfer",false)) {
      err=transfer(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"sync",false)) {
      err=synchronize(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"texport",false)) {
      err=texport(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"debitnote",false)) {
      err=debitnote(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"accadd",false)) {
      err=accAdd(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"getstdorders",false)) {
      err=getStandingOrders(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"newstdorder",false)) {
      err=newStandingOrder(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"delstdorder",false)) {
      err=delStandingOrder(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"resetseq",false)) {
      err=resetSeq(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"report",false)) {
      err=report(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"disablekeys",false)) {
      err=disableKeys(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"changekeys",false)) {
      err=changeKeys(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"modifymedium",false)) {
      err=modifyMedium(hbciif,opt);
    }
    else if (-1!=HBCI::parser::cmpPattern(cmd,"chgversion",false)) {
      err=chgVersion(hbciif,opt);
    }
    else {
      err=HBCI::Error("main()",
		      ERROR_LEVEL_NORMAL,
		      0,
		      ERROR_ADVISE_DONTKNOW,
		      "Unknown command",
		      cmd);
      rv=1;
    }
    if (!err.isOk()) {
      switch(err.code()) {
      case HBCI_ERROR_CODE_PIN_WRONG_0:
	fprintf(stderr,
		"VERY SERIOUS WARNING:\n"
		"=====================\n"
		"You already entered a false pin 3 times !\n"
		"If you enter a bad pin again "
		"your card gets blocked!\n"
		"Please use now the last program which allowed you "
		"to successfully enter the pin in order to reset the\n"
		"error counter !\n");
	break;
      case HBCI_ERROR_CODE_PIN_WRONG_1:
	fprintf(stderr,
		"SERIOUS WARNING:\n"
		"=====================\n"
		"You already entered a false pin twice !\n"
		"There is only one bad try left, so please be carefull.\n"
		"Please use now the last program which allowed you "
		"to successfully enter the pin in order to reset the\n"
		"error counter !\n");
	break;
      case HBCI_ERROR_CODE_PIN_WRONG_2:
	fprintf(stderr,
		"WARNING:\n"
		"=====================\n"
		"You entered a false pin.\n"
		"There are only two bad tries left, "
		"so please be carefull.\n"
		"If you are very sure that the pin you entered is "
		"correct \n"
		"then please stop using this program and contact\n"
		"         martin@libchipcard.de \n"
		"to report this incidence.\n"
		"If it is possible that you made a mistake, then please\n"
		"try again.\n");
	break;
      case HBCI_ERROR_CODE_PIN_WRONG:
	fprintf(stderr,
		"WARNING:\n"
		"=====================\n"
		"You entered a false pin.\n"
		"If you are very sure that the pin you entered is "
		"correct \n"
		"then please stop using this program and contact\n"
		"         martin@libchipcard.de \n"
		"to report this incidence.\n"
		"If it is possible that you made a mistake, then please\n"
		"try again.\n");
	break;
      default:
	fprintf(stderr,"ERROR: %s\n",
		err.errorString().c_str());
      } // switch
    }
    else
      rv=0;
  } // try
  catch (HBCI::Error lerr) {
    fprintf(stderr,"Exception: %s\n",
	    lerr.errorString().c_str());
    rv=0x8;
  }
#ifdef HAS_LIBCHIPCARD
  catch (CTError lerr) {
    fprintf(stderr,"CT-Exception: %s\n",
	    lerr.errorString().c_str());
    rv=0x8;
  }
#endif
  try {
    // save environment
    err=hbciif.ref().saveEnvironment(cfgfile);
    hbciif=0;
  } // try
  catch (HBCI::Error lerr) {
    err=lerr;
  }

  if (!err.isOk()) {
    fprintf(stderr,"ERROR: %s\n",
	    err.errorString().c_str());
    rv|=0x04;
  }

  return rv;
}


int main(int argc, char **argv) {
  int result;

  try {
    result=_main(argc,argv);
  }
  catch (HBCI::Error lerr) {
    cerr<<"Exception: "<<lerr.errorString()<<"\n";
  }
  return result;
}



